<?php

namespace Drupal\pwa_firebase;

/**
 * Service to send notification.
 */
class Firebase {

  /**
   * Function to send a notification to all the users.
   */
  public function sendMessageToAllUsers($title, $message, $url = NULL, $option = []) {

    $database = \Drupal::database();
    $tokens = $database->select('pwa_firebase', 'f')
      ->fields('f', ['token'])
      ->execute()
      ->fetchCol();
    // @todo send with option device type android.
    $this->sendNotification($tokens, $title, $message, $url, $option);
  }

  /**
   * Function to send a notification to 1 user.
   */
  public function sendMessageToUser($uid, $title, $message, $url = NULL, $option = []) {

    $database = \Drupal::database();
    $tokens = $database->select('pwa_firebase', 'f')
      ->fields('f', ['token'])
      ->condition('uid', $uid)
      ->execute()
      ->fetchCol();
    // @todo send with option device type android.
    $this->sendNotification($tokens, $title, $message, $url, $option);
  }

  /**
   * Function send message to firebase.
   */
  public function sendNotification($tokens, $title, $message, $url = NULL, $option = []) {
    global $base_url;
    global $base_path;

    $config = \Drupal::config('pwa_firebase.settings');
    if (strlen($title) > 255) {
      $title = substr($title, 0, 254) . '…';
    }
    if (strlen($message) > 1024) {
      $message = substr($message, 0, 1023) . '…';
    }
    if (empty($url)) {
      $url = $base_url . $base_path;
    }

    $headers = [
      'Content-Type: application/json',
      "Accept: application/json",
      'Authorization: key=' . $config->get('firebase_server_key'),
    ];

    // Note: this is the image that is used for the manifest file.
    $image = !empty($option['image']) ? $base_url . $base_path . $option['image'] : 'optional-image';

    $param = [
      'badge' => !empty($option['badge']) ? $option['badge'] : 1,
      'icon' => $image,
      'link' => $url,
      'sound' => 'optional-sound',
      'priority' => !empty($option['priority']) ? $option['priority'] : 'high',
      'tag' => !empty($option['tag']) ? $option['tag'] : '',
      'bg' => !empty($option['bg']) ? $option['bg'] : 'info',
      // Background boostrap danger|info|warning|primary|success.
      'vibrate' => 1,
      'sub_title' => !empty($option['sub_title']) ? $option['sub_title'] : '',
      'type' => !empty($option['sub_title']) ? $option['sub_title'] : 1,
      'activitydata' => !empty($option['activitydata']) ? $option['activitydata'] : NULL,
    ];
    if (!empty($option['device_type']) && $option['device_type'] == 'android') {
      $option['content_available'] = TRUE;
      $option['priority'] = "high";
    }

    \Drupal::moduleHandler()->alter('pwa_firebase_send', $param);
    ini_set("allow_url_fopen", "On");
    $data = [
      "content_available" => !empty($option['content_available']) == 1 ? FALSE : TRUE,
      "priority" => !empty($option['priority']) ? $option['priority'] : 'high',
      "registration_ids" => $tokens,
      "notification" => [
        "text" => $title,
        'body' => $message,
        'title' => $title,
        'alert' => $message,
        'sound' => 'shutter.wav',
      ],
      "data" => $param,
    ];

    $options = [
      'http' => [
        'method' => 'POST',
        'content' => json_encode($data),
        'header' => implode("\r\n", $headers),
      ],
    ];
    $context = stream_context_create($options);
    $result = json_decode(file_get_contents($config->get('firebase_endpoint'), FALSE, $context));
    if (!empty($result->failure)) {
      $database = \Drupal::database();
      foreach ($result->results as $index => $value) {
        if (!empty($value->error)) {
          $database->delete('pwa_firebase')
            ->condition('token', $tokens[$index])
            ->execute();
        }
      }
    }
    return $result;
  }

}
